local screen
local lock = 0.6
local actorSelf
local current = 1

local configList = {
    {Key = 'MemoryCards', List = {false, true}, Default = false},
    {Key = 'CustomSongsEnable', List = {false, true}, Default = false},
    {Key = 'ShowThemeErrors', List = {false, true}, Default = false},
    {Key = 'SongBackgrounds', List = {false, true}, Default = false},
    {Key = 'RateModsAffectFGChanges', List = {false, true}, Default = true},
    {Key = 'CustomShadersDisabled', List = {false, true}, Default = false},
    {Key = 'DrawHoldsAndTapsInSameLoop', List = {false, true}, Default = false},
    {Key = 'GimmickMode', List = {false, true}, Default = true},
    {Key = 'AllowVRR', List = {false, true}, Default = true},
}

for k,v in pairs(configList) do
    if PREFSMAN:PreferenceExists(v.Key) then
        configList[k].Value = PREFSMAN:GetPreference(v.Key)
        configList[k].Select = 1
        for index, value in pairs(v.List) do
            if value == configList[k].Value then
                configList[k].Select = index
                break
            end
        end
    else
        -- そもそも項目が存在しない（非対応本体）の場合は項目から消す
        configList[k] = nil
    end
end
W_CUSTOM:SV('configList', configList)

-- キー・マウス操作
local function CallbackInput(params)
    local function CheckButton(params, key)
        return (params.button and params.button == key)
            or (params.GameButton and params.GameButton == key)
    end

    if params.type == 'InputEventType_Release' then
        if CheckButton(params, 'Back') then
            actorSelf:playcommand('EndCustom')
        end
        if CheckButton(params, 'Start') then
            -- 保存の場合は保存処理、それ以外の場合は最下部へ
            if current < #configList+1 then
                current = #configList+1
                actorSelf:playcommand('Cursor')
            else
                actorSelf:playcommand('Save')
            end
        end
    end
    if params.type == 'InputEventType_FirstPress' or params.type == 'InputEventType_Repeat' then
        if CheckButton(params, 'Up') then
            if current > 1 then
                current = current-1
            else
                current = #configList+1
            end
            actorSelf:playcommand('Cursor')
        end
        if CheckButton(params, 'Down') then
            if current < #configList+1 then
                current = current+1
            else
                current = 1
            end
            actorSelf:playcommand('Cursor')
        end
        if CheckButton(params, 'Left') then
            if current > #configList then return end
            if configList[current].Select > 1 then
                configList[current].Select = configList[current].Select-1
            else
                configList[current].Select = #configList[current].List
            end
            configList[current].Value = configList[current].List[configList[current].Select]
            actorSelf:playcommand('Change')
        end
        if CheckButton(params, 'Right') then
            if current > #configList then return end
            if configList[current].Select < #configList[current].List then
                configList[current].Select = configList[current].Select+1
            else
                configList[current].Select = 1
            end
            configList[current].Value = configList[current].List[configList[current].Select]
            actorSelf:playcommand('Change')
        end
    end
end

return Def.ActorFrame({
    InitCommand = function(self)
        actorSelf = self
    end,
    OnCommand = function(self)
        MESSAGEMAN:Broadcast('ScrollMenu', {Cursor = current, Init = true})
        screen = SCREENMAN:GetTopScreen()
        screen:lockinput(lock)
        screen:AddInputCallback(CallbackInput)
        MESSAGEMAN:Broadcast('UpdateHelper', {Text = W_CUSTOM:String('Helper', 'Profile')})
    end,
    -- 項目を選択
    CursorCommand = function(self)
        self:playcommand('PlayCursorSound')
        MESSAGEMAN:Broadcast('ChangedCursor', {Index = current})
    end,
    -- 設定を変更
    ChangeCommand = function(self)
        self:playcommand('PlayCursorSound')
        MESSAGEMAN:Broadcast('UpdateValue', {Index = current, Value = configList[current].Value})
    end,
    -- 設定を保存
    SaveCommand = function(self)
        self:playcommand('PlayStartSound')
        for _,v in pairs(configList) do
            if PREFSMAN:PreferenceExists(v.Key) then
                PREFSMAN:SetPreference(v.Key, v.List[v.Select])
            end
        end
        self:playcommand('EndCustom', {SE = false})
    end,
    W_CUSTOM:Template('Basic')
})