-- ライフ
local pn = ...
local screen
local life
local battery
local tmpLife = 0
local width  = 240
local height = 12

local Border = LoadActor(W:Object('radius'), {
    Width  = width,
    Height = height,
    Radius = height,
    Color  = color('1, 1, 1, 0.5'),
})
local BG = LoadActor(W:Object('radius'), {
    Width  = width-2,
    Height = height-2,
    Radius = height-2,
    Color  = color('0, 0, 0, 0.8'),
})
local Meter = LoadActor(W:Object('radius'), {
    Id     = 'Meter',
    Width  = width-4,
    Height = height-4,
    Radius = height-4,
    Color  = color('0, 0, 0, 1'),
})
local Over = LoadActor(W:Object('radius'), {
    Id     = 'Over',
    Width  = width-4,
    Height = height-4,
    Radius = height-4,
    Color  = color('1, 1, 1, 1'),
})

-- バッテリーライフの境界線（ライフ数が11以上の場合は表示しない）
local function BorderActor()
    local actors = Def.ActorFrame({})
    for i=1, 9 do
        actors[#actors+1] = Def.Quad({
            OnCommand = function(self)
                self:zoomto(2, height)
                self:diffuse(0.5, 0.5, 0.5, 1)
                self:fadetop(0.5)
                self:fadebottom(0.5)
            end,
            ChangedPlayerLifeMessageCommand = function(self, params)
                if not battery or battery > 10 then
                    self:x(0)
                    self:visible(false)
                else
                    self:x(-width/2 + width*i/battery + 1)
                    self:visible(i<battery)
                end
            end,
        })
    end
    return actors
end

local lifeColor = nil
local barWidth = 0
return Def.ActorFrame({
    OnCommand = function(self)
        self:addx((W:IsP1(pn) and -1 or 1) * 100)
        self:diffusealpha(0)
        self:sleep(W_TIMER:GameStartSleep()+0.3)
        self:smooth(0.5)
        self:addx((W:IsP1(pn) and 1 or -1) * 100)
        self:diffusealpha(1)
    end,
    Def.ActorFrame({
        OnCommand = function(self)
            screen  = SCREENMAN:GetTopScreen()
            life    = screen:GetLifeMeter(pn) or nil
            battery = life.GetTotalLives and life:GetTotalLives() or nil
        end,
        ScreenUpdateMessageCommand = function(self, params)
            local curLife = life and life:GetLife() or 0
            if tmpLife == curLife then return end
            tmpLife = curLife
            self:playcommand('ChangedPlayerLife', {
                Player = pn,
                Life   = life and curLife or 0,
                Danger = life
                    and (battery and (life:GetLivesLeft() == 1) or life:IsInDanger())
                    or false,
            })
        end,
        Border:Actor(),
        BG:Actor(),
        Meter:Actor()..{
            InitCommand = function(self)
                self:horizalign(W:IsP1(pn) and left or right)
                self:x(W:IsP1(pn) and -width/2+2 or width/2-2)
            end,
            ChangedPlayerLifeMessageCommand = function(self, params)
                if params.Player == pn then
                    self:stoptweening()
                    barWidth = params.Life * (width-4)
                    local col = {}
                    if battery and life:GetTotalLives() == 1 then
                        col = {1, 0, 0, 1}
                    elseif lifeColor then
                        if lifeColor == 'rainbow' then
                            col = color('#e0e0e0ff')
                        else
                            col = color(lifeColor)
                        end
                    elseif params.Life >= 1 then
                        col = {0.8, 1, 0.5, 1}
                    elseif params.Danger then
                        col = {1, 0, 0, 1}
                    else
                        col = {0, 0.8, 1, 1}
                    end
                    self:playcommand('UpdateRadius', {
                        Id    = 'Meter',
                        Color = col,
                        Width = barWidth,
                        Wait  = 0.2,
                    })
                    self:linear(0.2)
                    self:x(W:IsP1(pn) and -width/2+2 + barWidth/2 or width/2-2 - barWidth/2)
                end
            end,
        },
        -- ゲージ
        Over:Actor()..{
            InitCommand = function(self)
                self:horizalign(W:IsP1(pn) and left or right)
                self:x(W:IsP1(pn) and -width/2+2 or width/2-2)
                self:diffuseshift()
                self:effectcolor1(1, 1, 1, 0.5)
                self:effectcolor2(1, 1, 1, 0)
                self:effectclock('bgm')
            end,
            ChangedPlayerLifeMessageCommand = function(self, params)
                if params.Player == pn then
                    self:stoptweening()
                    local barWidth = params.Life * (width-4)
                    self:playcommand('UpdateRadius', {
                        Id    = 'Over',
                        Width = barWidth,
                        Wait  = 0.2,
                    })
                    self:linear(0.2)
                    self:x(W:IsP1(pn) and -width/2+2 + barWidth/2 or width/2-2 - barWidth/2)
                end
            end,
        },
        BorderActor(),
        -- 数字
        LoadFont('Common Small Normal')..{
            InitCommand = function(self)
                self:diffuse(Color('White'))
                self:strokecolor(Color('Outline'))
                self:horizalign(W:IsP1(pn) and left or right)
                self:xy((W:IsP1(pn) and -1 or 1) * (width/2-2), height-2+3)
                self:zoom(0.5)
            end,
            ChangedPlayerLifeMessageCommand = function(self, params)
                if params.Player == pn then
                    self:stoptweening()
                    if battery then
                        self:settext(''..life:GetLivesLeft()..' / '..life:GetTotalLives())
                    else
                        if params.Life >= 1 then
                            self:settext('100%')
                        elseif params.Life <= 0 then
                            self:settext('0%')
                        else
                            self:settextf('%4.1f%%', params.Life*100)
                        end
                    end
                end
            end,
        },
        -- プレイヤー
        LoadFont('Common Small Normal')..{
            InitCommand = function(self)
                if GAMESTATE:IsPlayerEnabled(pn) then
                    local pleyerName = W_PLAYER:Name(pn)
                    self:diffuse(Color('White'))
                    self:strokecolor(Color('Outline'))
                    self:horizalign(W:IsP1(pn) and left or right)
                    self:xy((W:IsP1(pn) and -1 or 1) * (width/2-2), -height-2-1)
                    self:zoom(0.5)
                    self:settext(pleyerName)
                else
                    self:settext('')
                    self:visible(false)
                end
            end,
        },
        -- フレア
        LoadFont('Common Small Normal')..{
            InitCommand = function(self)
                self:diffuse(Color('White'))
                self:strokecolor(Color('Outline'))
                self:horizalign(W:IsP1(pn) and left or right)
                self:xy(0, height-2+3)
                self:zoom(0.5)
                self:settext('')
            end,
            ChangedLifeTypeCommand = function(self, params)
                if not params.Level then return end
                if params.Player == pn then
                    lifeColor = params.Color
                    self:settext(params.Name or '')
                end
            end,
        },
    }),
    -- ターゲットスコアでアバター表示するのでここは不要
    -- W_AVATAR:Actor(W_AVATAR:Path('sample.ini'), 'Avatar'..pname(pn), {
    --         Color = W_PLAYER:Color(pn),
    --         Flip  = W:IsP1(pn),
    --         Mini  = false,
    --     })..{
    --     InitCommand = function(self)
    --         self:zoom(0.35)
    --         self:xy((W:IsP1(pn) and -1 or 1) * ((width+15)/2), 0)
    --     end,
    -- },
})