local color = {
    PlayerNumber_P1  = color('#dd3322ff'),
    PlayerNumber_P2  = color('#2299eeff'),
}

-- 色を取得
local function GetColor(self, key)
    return color[key] or Color('White')
end

-- 名前を取得
local function Name(self, pn)
    local pleyerName =PROFILEMAN:GetPlayerName(pn)
    return (pleyerName and pleyerName ~= '')
        and pleyerName
        or 'Player'..(W:PNum(pn))
end

-- レーダーを取得
local function GetRadarValue(self, pn)
    local song = GAMESTATE:GetCurrentSong()
    local steps = GAMESTATE:GetCurrentSteps(pn)
    if not song or not steps or not pn then
        return {
            RadarCategory_Stream  = 0,
            RadarCategory_Voltage = 0,
            RadarCategory_Air     = 0,
            RadarCategory_Freeze  = 0,
            RadarCategory_Chaos   = 0,
        }
    end
    local radar = steps:GetRadarValues(pn)
    local lifts = radar:GetValue('RadarCategory_Lifts')
    local rolls = radar:GetValue('RadarCategory_Rolls')
    local mines = radar:GetValue('RadarCategory_Mines')
    local hands = radar:GetValue('RadarCategory_Hands')
    local values = {
        RadarCategory_Stream  = (radar:GetValue('RadarCategory_Stream') + (lifts + 0.5*rolls + 0.25*mines + 0.5*hands)*0.0008) * 100,
        RadarCategory_Voltage = (radar:GetValue('RadarCategory_Voltage')) * 100,
        RadarCategory_Air     = (radar:GetValue('RadarCategory_Air') + 0.005*hands) * 100,
        RadarCategory_Freeze  = (radar:GetValue('RadarCategory_Freeze') + 0.01*rolls) * 100,
        RadarCategory_Chaos   = (radar:GetValue('RadarCategory_Chaos') + 0.001*mines) * 100,
    }
    for k,v in pairs(values) do
        local convert = v
        if convert > 100 then
            if k == 'RadarCategory_Air' or k == 'RadarCategory_Freeze' then
                convert = 100 + (convert - 100)/12
            elseif k == 'RadarCategory_Voltage' or k == 'RadarCategory_Chaos' then
                convert = 100 + (convert - 100)/8
            else
                convert = 100 + (convert - 100)/4
            end
        end
        if convert > 100.5 then
            if k == 'RadarCategory_Voltage' then
                convert = 100 + (convert - 100)/2
            else
                convert = 100.5 + (convert - 100.5)/1
            end
        end
        if convert > 150 then
            if k == 'RadarCategory_Voltage' then
                convert = 150 + (convert - 160)/4
            elseif k == 'RadarCategory_Air' or k == 'RadarCategory_Freeze' then
                convert = 150 + (convert - 160)/3
            else
                convert = 150 + (convert - 160)/2
            end
        end
        if convert > 200 then
            convert = 200
        end
        values[k] = convert
    end
    return values
end

-- ステップゾーンのX位置
local function GetStepZonePosX(self, pn)
    local r = SCREEN_CENTER_X
    local cs = GAMESTATE:GetCurrentStyle()
    if cs then
        local st = cs:GetStyleType()
        if (GAMESTATE:GetNumPlayersEnabled() == 1 and Center1Player()) or cs:ColumnsPerPlayer() > 5 then
            r = SCREEN_CENTER_X
        else
            r = THEME:GetMetric('ScreenGameplay', 'Player'..pname(pn)..ToEnumShortString(st)..'X')
        end
    else
        r = THEME:GetMetric('ScreenGameplay', 'Player'..pname(pn)..'OnePlayerOneSideX')
    end
    return r
end

-- ステップゾーン座標未定義時のテンプレート
-- 外部呼出し不可
local ReceptorPosition
local function LoadReceptorPosition()
    ReceptorPosition = dofile(THEME:GetPathO('', 'ReceptorPositions.lua'))
    return ReceptorPosition
end
-- 現在の楽曲からテンプレートまたは定義を取得
-- 外部呼出し不可
local function GetReceptorPositionTable()
    if not ReceptorPosition then
        LoadReceptorPosition()
    end
    local rp = YA_GROUP:ReceptorPosition(GAMESTATE:GetCurrentSong())
    -- 値がテーブルではない場合、receptorPositionで定義しているかチェック
    local rpTable
    -- フォルダ単位での定義
    if type(rp) == 'string' and ReceptorPosition(rp) then
        rpTable = ReceptorPosition(rp)
    -- フォルダ単位での定義あり（テーブル定義）
    elseif type(rp) == 'table' then
        -- 未定義データとマージ
        -- rpTable = ReceptorPosition('')
        -- この場合キー名が被ると元の値が上書きされるのでループで取得すること
        rpTable = {}
        for xy,xyVal in pairs(ReceptorPosition('')) do
            rpTable[xy] = {}
            for key,v in pairs(xyVal) do
                if rp[xy] and rp[xy][key] then
                    rpTable[xy][key] = rp[xy][key]
                else
                    rpTable[xy][key] = v
                end
            end
        end
    -- Group.ini/luaに定義なし
    else
        rpTable = ReceptorPosition('')
    end
    return rpTable
end

-- ステップゾーンの画面中心からのY座標を設定して取得
local function GetStepZonePosY(self, scroll)
    local r = 0
    local rpTable = GetReceptorPositionTable()
    -- テーブルの定義か、receptorPositionに存在する必要あり
    if rpTable and rpTable.Y and rpTable.Y[scroll] then
        r = rpTable.Y[scroll]
    end
    -- 6パネル二人プレイのように矢印が縮小されている場合はBGAの方でずらす処理を入れる
    return r
end

-- ステップゾーンのX座標を設定
local function SetStepZonePosX(self, plasyerSide)
    local r = SCREEN_CENTER_X
    local rpTable = GetReceptorPositionTable()
    -- テーブルの定義か、receptorPositionに存在する必要あり
    if not rpTable or not rpTable.X then return r end
    if rpTable and rpTable.X[plasyerSide] then
        r = rpTable.X[plasyerSide]
    end
    -- Metricsを再読み込み
    THEME:ReloadMetrics()
    return r
end

-- ステップゾーンの横幅（大体の値）
local function GetStepZoneWidth(self, pn)
    return GAMESTATE:GetCurrentStyle():GetWidth(pn)
end

-- ステップゾーンの横幅（オプションを考慮した）
local function GetStepZoneRealWidth(self, pn)
    local one = THEME:GetMetric('ArrowEffects', 'ArrowSpacing')
    local child = SCREENMAN:GetTopScreen():GetChild('Player'..pname(pn))
    local po = GAMESTATE:GetPlayerState(pn):GetCurrentPlayerOptions()
    local mini = 1.0 - (po:Mini()/2) * 1.0 - (po:Tiny()/2)    -- Tinyは幅だけ変わる（スクロールスピードはそのまま）
    local r = one + (GetStepZoneWidth(self, pn) - one) * math.abs(0.5 - po:Flip())*2
    return r * mini * (child and child:GetZoomX() or 128)
end

-- プロファイル種別
local function GetProfileType(self, pn)
    -- なんかゲストでもNormalで取得されることがあるのでこっちでチェック
    if not PROFILEMAN:IsPersistentProfile(pn) then
        return 'ProfileType_Guest'
    end
    return PROFILEMAN:GetProfile(pn):GetType()
end

-- ユーザーの設定保存フォルダを取得
local function GetUserPrefsPath(self, ...)
    local pn = ...
    local path = ''
    if pn and GetProfileType(self, pn) == 'ProfileType_Normal' then
        path = PROFILEMAN:GetProfileDir('ProfileSlot_Player'..W:PNum(pn))
    else
        path = 'Save/'
    end
    return path..'UserPrefs/'..THEME:GetCurThemeName()..'/'
end

-- ファイル処理
local function FilePref(self, file, mode, ...)
    local defOrValue, forceFlash = ...
    if mode == 1 and not FILEMAN:DoesFileExist(file) then
        return (defOrValue ~= nil) and defOrValue or nil
    end
    local ret = ''
    local f = RageFileUtil.CreateRageFile()
    if f:Open(file, mode) then
        if mode == 2 then
            f:Write(defOrValue)
            ret = defOrValue
        else
            ret = f:GetLine()
        end
        f:Close()
    end
    f:destroy()
    if mode == 2 and (forceFlash == nil or forceFlash == true) then
        W:Flush(file)
    end
    if ret == nil and defOrValue ~= nil then
        return defOrValue
    end
    if ret == 'TRUE' or ret == 'FALSE' then
        ret = (ret == 'TRUE')
    end
    return ret
end

-- プレイヤーの設定ファイルを操作
local function PnFilePref(self, pn, key, mode, ...)
    if pn and not GAMESTATE:IsPlayerEnabled(pn) then
        return nil
    end
    local suffix = (not pn or GetProfileType(self, pn) == 'ProfileType_Normal') and '' or pname(pn)
    local file = GetUserPrefsPath(self, pn)..key..suffix
    return FilePref(self, file, mode, ...)
end

-- PROFILE IDの設定ファイルを操作
local function ProfileFilePref(self, id, key, mode, ...)
    local dir = PROFILEMAN:LocalProfileIDToDir(id)
    local file = dir..'UserPrefs/'..THEME:GetCurThemeName()..'/'..key
    return FilePref(self, file, mode, ...)
end

-- 保存
local function SavePref(self, pn, key, value)
    if type(value) == 'boolean' then
        value = value and 'TRUE' or 'FALSE'
    end
    PnFilePref(self, pn, key, 2, value)
end

-- 読み込み
local function ReadPref(self, pn, key, ...)
    local ret = PnFilePref(self, pn, key, 1, ...)
    return ret
end

-- JOINかつ有効プレイヤーかどうか
local function IsPlayerJoined(self, pn)
    return GAMESTATE:IsPlayerEnabled(pn) and GAMESTATE:IsSideJoined(pn)
end

-- JOINかつ有効プレイヤー人数
local function IsNumJoined(self)
    return (IsPlayerJoined(self, PLAYER_1) and 1 or 0)
        + (IsPlayerJoined(self, PLAYER_2) and 1 or 0)
end

-- PROFILEパスからアバターを取得
local function GetAvatarFromPath(self, path)
    if not path or path == '' then
        return nil, 'None'
    end
    -- WAIEIの設定ファイルから読みこむ
    -- SelectProfileでも表示させる必要があるのでファイルパスは直接指定
    local file = W_STRING:ReadFile(path..'UserPrefs/'..THEME:GetCurThemeName()..'/Avatar')
    if file ~= '' then
        return W_AVATAR:Path(file), 'WAIEI'
    end
    -- CyberiaStyleアバター
    file = W_GRAPHICS:GetFile(path..'CSAvatar')
    if file and FILEMAN:DoesFileExist(file) then
        return file, 'CS'
    end
    -- OutFoxアバター
    file = W_GRAPHICS:GetFile(path..'icon')
    if file and FILEMAN:DoesFileExist(file) then
        return file, 'OutFox'
    end
    return nil, 'None'
end

-- プレイヤーのアバターを取得
local function GetPlayerAvatar(self, pn)
    -- マシンプロファイルはアバター無し
    if GetProfileType(self, pn) == 'ProfileType_Guest' then
        return nil, 'Guest'
    end
    local path = ''
    if pn and GetProfileType(self, pn) ~= 'ProfileType_Guest' then
        path = PROFILEMAN:GetProfileDir('ProfileSlot_Player'..W:PNum(pn))
    else
        path = 'Save/'
    end
    return GetAvatarFromPath(self, path)
end

-- ProfileIndexのアバターを取得
local function GetIndexAvatar(self, ...)
    local index = ...
    -- indexが0未満または未設定の場合はマシンプロファイルとして扱う
    if not index or index < 0 or index >= PROFILEMAN:GetNumLocalProfiles() then
        return nil, 'Guest'
    end
    return GetAvatarFromPath(self,
        PROFILEMAN:LocalProfileIDToDir(
            PROFILEMAN:GetLocalProfileIDFromIndex(index)
        ))
end

-- ゲーム中のプレイヤーにEDIT譜面があるかどうか
local function IsSomeoneEdit()
    local isEdit = false
    for p,pn in pairs(PlayerNumber) do
        if GAMESTATE:IsPlayerEnabled(pn) then
            local steps = W:Steps(pn)
            if steps and (steps:IsAnEdit() or steps:IsAPlayerEdit()) then
                isEdit = true
                break
            end
        end
    end
    return isEdit
end

-- オプションをテキストで取得
local function GetPlayerOptionString(self, pn)
    local ps = GAMESTATE:GetPlayerState(pn)
    -- なぜか言語ファイルが機能していない？
    local optionString = string.gsub(ps:GetPlayerOptionsString('ModsLevel_Stage'), 'ImmediateContinue', 'Delayed')
    -- 継続コンボがGOOD以上でバッテリーライフの場合
    local minCombo = W_OPTION:ReadPref('ContinueCombo', 'TapNoteScore_W4')
    if minCombo ~= 'TapNoteScore_W4' then
        optionString = string.gsub(optionString, '(%d+Lives)', string.upper(W_JUDGMENT:Name(minCombo))..' %1')
        optionString = string.gsub(optionString, '1Lives', 'Fullcombo Attack')
    end
    local gimmick = getenv('WAIEI_GIMMICK_PLAYER') or {[PLAYER_1] = '', [PLAYER_2] = ''}
    if gimmick[pn] and gimmick[pn] ~= '' and gimmick[pn] ~= 'Off' then
        optionString = optionString..', '..gimmick[pn]
        local gimmickConfig = getenv('WAIEI_GIMMICK_CONFIG') or {[PLAYER_1] = 'Off', [PLAYER_2] = 'Off'}
        if gimmickConfig[pn] and gimmickConfig[pn] ~= 'Off' then
            optionString = optionString..'['..gimmickConfig[pn]..']'
        end
    end
    return optionString
end

-- リバースかどうか
local function PlayerIsReverse(self, pn)
    local ps = GAMESTATE:GetPlayerState(pn)
    local pop = ps:GetPlayerOptions("ModsLevel_Preferred")
    local reverse = ({pop:Reverse()})[1]
    return (reverse >= 0.5)
end

-- PROFILE一覧を取得
local function GetProfileList(self)
    local profiles = {}
    for k,v in pairs(PROFILEMAN:GetLocalProfileIDs()) do
        local profile = PROFILEMAN:GetLocalProfile(v)
        profiles[#profiles+1] = {
            Id   = v,
            Guid = profile:GetGUID(),
            Name = profile:GetDisplayName(),
        }
    end
    table.sort(profiles, function(a, b)
        return a.Id < b.Id
    end)
    return profiles
end

-- BPMの最小最大を取得
local function GetMinMaxBpms(steps)
    -- DisplayBPMが指定されていたらそれを優先（0だろうとそれを表示）
    return (steps:GetDisplayBPMType() == 'DisplayBPM_Specified')
                    and steps:GetDisplayBpms()
                    or steps:GetTimingData():GetActualBPM()
end

-- 画面に表示するBPM
local function HighspeedBPM(self, pn, mod, speed, ...)
    local steps = ...
    steps = steps or W:Steps(pn)
    if not steps or steps:GetDisplayBPMType() == 'DisplayBPM_Random' then
        return '??? - ???'
    end
    local bpms = GetMinMaxBpms(steps)
    mod = string.lower(mod)
    if mod == 'c' then
        return string.format('%d - %d', speed, speed)
    elseif mod == 'm' then
        local max = tonumber(THEME:GetMetric('Player', 'MModHighCap'))
        if bpms[2] > max then
            speed = speed / max
        else
            speed = (bpms[2] ~= 0) and speed / bpms[2] or 0
        end
    -- elseif mod == 'a' then
    --     local avg = po:AvarageScrollBPM()
    --     local time = avg / speed
    --     return string.format('%d - %d', math.round(bpms[1] * time), math.round(bpms[2] * time))
    elseif mod ~= 'x' then
        return string.format('(%s)??? - ???', mod)
    end
    if bpms[2] > 99999 then
        return string.format('%d - 99999+', math.round(bpms[1] * speed))
    end
    return string.format('%d - %d', math.round(bpms[1] * speed), math.round(bpms[2] * speed))
end

-- 平均BPM
local function CalcAverageBPM(self, pn, mod, speed, ...)
    local steps = ...
    steps = steps or W:Steps(pn)
    if not steps or steps:GetDisplayBPMType() == 'DisplayBPM_Random' then
        return -1
    end
    if mod == 'C' then
        return 1.0
    end
    local song = W:Song()
    if not song then
        return -1
    end
    local tm = steps:GetTimingData()
    local total = 0
    local bpms = {}
    for _,v in pairs(tm:GetBPMsAndTimes()) do
        bpms[#bpms+1] = {tonumber(split('=', v)[1]), tonumber(split('=', v)[2])}
    end
    local lastBeat = tm:GetBeatFromElapsedTime(song:GetLastSecond())
    bpms[#bpms+1] = {lastBeat, bpms[#bpms][2]}
    for i = 1, #bpms-1 do
        local sec = 1.0 * tm:GetElapsedTimeFromBeat(bpms[i+1][1]) - tm:GetElapsedTimeFromBeat(bpms[i][1])
        total = 1.0 * total + sec * bpms[i][2]
    end
    local stopTotal = 0
    for _,v in pairs(tm:GetStops()) do
        stopTotal = 1.0 * stopTotal + tonumber(split('=', v)[2])
    end
    for _,v in pairs(tm:GetDelays()) do
        stopTotal = 1.0 * stopTotal + tonumber(split('=', v)[2])
    end
    local denominator = song:GetLastSecond() - tm:GetElapsedTimeFromBeat(0) - stopTotal
    local res = (denominator > 0) and total / denominator or 0
    if mod == 'm' then
        local maxBpm = steps:GetTimingData():GetActualBPM()[2]
        local maxCap = tonumber(THEME:GetMetric('Player', 'MModHighCap'))
        if maxBpm > maxCap then
            speed = speed / maxCap
        else
            speed = (maxBpm ~= 0) and speed / maxBpm or 0
        end
        res = res * speed
    end
    return res
end

local function GetStepsBeginAndEnd(self, steps)
    local song = SONGMAN:GetSongFromSteps(steps)
    if not song then return {} end
    local path = song:GetSongFilePath()
    local extension = string.find(path, '.+%.ssc') and 'ssc' or (string.find(path, '.+%.sm') and 'sm' or nil)
    if not extension then return {
        Time = {Begin = song:GetFirstSecond(), End = song:GetLastSecond()},
        Beat = {Begin = song:GetFirstBeat(), End = song:GetLastBeat()}
    } end
    local timing     = steps:GetTimingData()
    local stepType   = string.upper(string.gsub(ToEnumShortString(steps:GetStepsType()), '_', '-'))
    local difficulty = string.upper(ToEnumShortString(steps:GetDifficulty()))
    local warps      = {}
    for _,v in pairs(timing:GetWarps() or {}) do
        local splitWarp = split('=', v)
        warps[#warps+1] = {tonumber(splitWarp[1]), tonumber(splitWarp[2])}
    end
    local warpIndex = 1

    local result = {Time = {Begin = -1, End = -1}, Beat = {Begin = -1, End = -1}}
    local file = RageFileUtil:CreateRageFile()
    file:Open(path, 1)
    file:Seek(0)
    -- Steps取得のためのチェック用変数
    local hasStepType   = false
    local hasDifficulty = false
    local discription = ''
    local line = ''         -- 1行分のデータ
    local lineData = {}     -- 1行分のデータを1小節分まとめる
    -- 譜面情報の読み取り
    local RNM = {HEADER = 1, PROPERTIES = 2, NOTE = 3}
    local readNoteMode = RNM.HEADER
    local currentBeat = 0
    local currentSec = 0
    while true do
        line = string.upper(file:GetLine())
        if readNoteMode == RNM.PROPERTIES then
            -- ノート先頭の譜面情報部分であれば飛ばす
            if not string.find(line, '^.+:') then
                readNoteMode = RNM.NOTE
            end
        end
        if readNoteMode == RNM.NOTE then
            -- 小節の区切り、ノートの終了
            if string.find(line, '[,;].*') then
                -- 1行の拍数
                local oneBeat = 4.0 / #lineData
                -- 1行ずつチェック
                for _,v in pairs(lineData) do
                    -- ワープ内の矢印は無視
                    local isWarp = false
                    local latestWarp = -1
                    for i=warpIndex, #warps do
                        if warps[i][1] <= currentBeat
                            and currentBeat < warps[i][1] + warps[i][2]
                            and warps[i][1] ~= latestWarp then
                            isWarp = true
                            break
                        end
                        if warps[i][1] > currentBeat then
                            break
                        end
                        warpIndex = i
                        latestWarp = warps[i][1] + warps[i][2]
                    end
                    currentSec = timing:GetElapsedTimeFromBeat(currentBeat)
                    -- positionバーは0秒から開始なのでそれより前のbeatは無視する
                    if currentSec >= 0 then
                        -- 現在の行がワープじゃないときだけカウント処理
                        if not isWarp then
                            if string.find(v, '[^0]') then
                                result = {
                                    Time = {
                                        Begin = result.Time.Begin < 0 and currentSec or result.Time.Begin,
                                        End = currentSec,
                                    },
                                    Beat = {
                                        Begin = result.Beat.Begin < 0 and currentSec or result.Beat.Begin,
                                        End = currentBeat,
                                    },
                                }
                            end
                        end
                    end
                    currentBeat = currentBeat+oneBeat
                end
                lineData = {}
                -- ノートの終了
                if string.find(line, '[;].*') then
                    break
                end
            end
            if not (line == ',') and not (line == ';') and not (line == ':') then
                lineData[#lineData+1] = split('/', line)[1]
            end
        else
            if extension == 'ssc' then
                -- EDITの時はエディット名も見る
                if string.find(line, '#DESCRIPTION:', 1, true) then
                    discription = string.gsub(line, "^.+:(.*);$", "%1")
                end
                -- スタイルを発見
                if string.find(line, '#STEPSTYPE:'..stepType..';', 1, true) then
                    hasStepType = true
                end
                -- 難易度を発見
                if string.find(line, '#DIFFICULTY:'..difficulty..';', 1, true) then
                    hasDifficulty = true
                end
                -- ノートデータを発見
                if string.find(line, '#NOTES:', 1, true) then
                    -- スタイル、難易度両方が見つかっている場合は取得対象
                    if hasStepType and hasDifficulty then
                        -- ただしEDITの場合はCHARTNAMEもチェック
                        if steps:IsAnEdit() or steps:IsAPlayerEdit() then
                            if (string.upper(steps:GetDescription()) == discription) then
                                readNoteMode = RNM.PROPERTIES
                            end
                        else
                            readNoteMode = RNM.PROPERTIES
                        end
                    end
                    hasStepType   = false
                    hasDifficulty = false
                end
            elseif extension == 'sm' then
                -- スタイルを発見
                if string.find(line, '%s*'..string.gsub(stepType, '-', '%%-')..':.*') then
                    hasStepType = true
                end
                -- 難易度を発見
                if string.find(line, '%s*'..difficulty..':.*') then
                    hasDifficulty = true
                    -- EDITの時はエディット名も見る
                elseif hasStepType and not hasDifficulty then
                    discription = string.gsub(line, "^%s*(.*):.*", "%1")
                end
                -- ノートデータを発見
                if string.find(line, '#NOTES:', 1, true) then
                    hasStepType   = false
                    hasDifficulty = false
                end
                -- スタイル、難易度両方が見つかっている場合は取得対象
                if hasStepType and hasDifficulty then
                    -- ただしEDITの場合はCHARTNAMEもチェック
                    if steps:IsAnEdit() or steps:IsAPlayerEdit() then
                        if (string.upper(steps:GetDescription()) == discription) then
                            readNoteMode = RNM.PROPERTIES
                        end
                    else
                        readNoteMode = RNM.PROPERTIES
                    end
                end
            end
        end
        -- 終端に到達した場合チェック終了
        if file:AtEOF() then
            break
        end
    end
    file:Close()
    file:destroy()
    return result
end

return {
    -- 色を取得
    Color        = GetColor,
    -- 名前を取得
    Name          = Name,
    -- グルーヴレーダーの値を取得
    Radar         = GetRadarValue,
    -- ステップゾーンのX座標を取得
    StepZoneX     = GetStepZonePosX,
    -- ステップゾーンのY座標を設定して返却
    StepZoneY     = GetStepZonePosY,
    -- ステップゾーンの幅を取得
    StepZoneW     = GetStepZoneWidth,
    -- オプションを考慮したステップゾーンの幅を取得
    StepZoneRealW = GetStepZoneRealWidth,
    -- ステップゾーンのX座標を設定
    SetStepZoneX  = SetStepZonePosX,
    -- プレイヤーがJOIN常態か
    IsEnabled     = IsPlayerJoined,
    -- JOIN状態のプレイヤー数を取得
    NumJoined     = IsNumJoined,
    -- ProfileTypeを取得
    ProfileType   = GetProfileType,
    -- プレイヤーの設定保存フォルダのパスを取得
    PrefsPath     = GetUserPrefsPath,
    -- プレイヤー単位で設定を保存
    SavePref      = SavePref,
    -- プレイヤー単位の設定を読み込み
    ReadPref      = ReadPref,
    -- PROFILE単位の設定を操作
    ProfilePref   = ProfileFilePref,
    -- プレイヤーのアバターパスを取得
    PlayerAvatar  = GetPlayerAvatar,
    -- 指定したインデックスのPROFILEのアバターパスを取得
    IndexAvatar   = GetIndexAvatar,
    -- 全プレイヤーが選択した譜面にエディットデータが含まれているか
    HasEdit       = IsSomeoneEdit,
    -- ゲームプレイ中のオプション
    GameplayMods  = GetPlayerOptionString,
    -- リバースかどうか
    IsReverse     = PlayerIsReverse,
    -- PROFILE一覧を取得
    List          = GetProfileList,
    -- ハイスピードを考慮したBPM
    HighspeedBPM  = HighspeedBPM,
    -- 平均BPM
    AverageBPM    = CalcAverageBPM,
    -- 譜面の開始と終了ビート、時刻を取得
    StepsBeginAndEnd = GetStepsBeginAndEnd,
}

--[[
Player.lua

Copyright (c) 2021 A.C

This software is released under the MIT License.
https://opensource.org/licenses/mit-license.php
--]]
